// ExamplePlugin.cpp : Defines the exported functions for the MysticThumbs plugin.
//

#include "ExamplePlugin.h"
#include "MysticThumbsPlugin.h"
#include <Shlwapi.h>
#include <math.h>
 
// Define a name and the extensions supported by this plugin
static const LPCWSTR s_name = L"Example Plugin";
static const LPCWSTR s_extensions[] = { L".mtp" };

#ifndef PI
#define PI 3.14159265358979f
#endif

class CExamplePlugin : public IMysticThumbsPlugin
{
	virtual LPWSTR GetName()
	{
		return StrDupW(s_name);
	}

	virtual void Destroy()
	{
		// Delete this instance
		delete this;
	}

	virtual bool PingImage(IN IStream* pStream, OUT MysticThumbsPluginPing& ping)
	{
		// Any ping members untouched will be ignored
		ping.bitDepth = 32;
		return true;
	}

	virtual unsigned char* GenerateImage(IN IStream* pStream, IN unsigned int desiredSize, IN unsigned int flags, OUT bool& hasAlpha, OUT unsigned int& width, OUT unsigned int& height)
	{
		// pStream is pointing to the head of the file stream on entry so you can begin by reading the file header etc.

		// for this example we are just going to generate a test-pattern image of size desiredSize x desiredSize, with transparency
		width = desiredSize;
		height = desiredSize;
		hasAlpha = true;
		const unsigned int numChannels = 4; // We are outputting alpha, so 4 channels

		// Allocate the image memory, Use LocalAlloc for this
		unsigned char* image = (unsigned char*)LocalAlloc(GMEM_FIXED, width * height * numChannels);

		// Decode
		unsigned char* ptr = image;
		for(unsigned int y=0; y<height; ++y)
		{
			float fy = ((float)y / (height-1)) * 2 - 1;
			for(unsigned int x=0; x<width; ++x)
			{
				// For this example, calculate some RGBA values since we don't actually have a real file stream
				// Instead of calculating values here we would normally read from pStream to gather the image data
				float fx = ((float)x / (width-1)) * 2 - 1;
				float r = (1+sinf(fx*PI))*0.5f;
				float g = (1+cosf(fy*PI))*0.5f;
				float b = (1+sinf(fx*fy*PI))*0.5f;
				float a = sqrtf(fx*fx + fy*fy);

				// Fill in the channel information for this pixel
				// Instead of this:
				*ptr++ = (unsigned char)(r * 255);
				*ptr++ = (unsigned char)(g * 255);
				*ptr++ = (unsigned char)(b * 255);
				*ptr++ = (unsigned char)(a * 255);
			}
		}

		return image;

		// return NULL if there is an error
		// make sure to set hasAlpha, width and height on successful completion
	}

	virtual unsigned int GetExtensionCount()
	{
		return ARRAYSIZE(s_extensions);
	}

	virtual LPWSTR GetExtension(IN unsigned int index)
	{
		return StrDupW(s_extensions[index]);
	}
};




// Initialize the plugin. Perform any work here that needs to be done for all instances such as loading DLLs or initializing globals.
extern "C" EXAMPLEPLUGIN_API bool Initialize()
{
	// For this example we don't need to do anything
	return true;
}

// Shutdown the plugin. Clean everything up.
extern "C" EXAMPLEPLUGIN_API bool Shutdown()
{
	// For this example we don't need to do anything
	return true;
}

// Create an instance of a thumbnail plugin
extern "C" EXAMPLEPLUGIN_API IMysticThumbsPlugin* CreateInstance()
{
	// Note that the IMysticThumbsPlugin::Destroy method takes care of deleting this object when MysticThumbs is finished with it.
	return new CExamplePlugin();
}
